/**********************************************************************
 * queue_func.c                                            October 1999
 * Horms                                             horms@valinux.com.
 * VA Linux System                                 http://valinux.com./
 *
 * Functions build around the queue ADT
 *
 * Licenced under the GNU GPL
 *
 **********************************************************************/

#include "queue_func.h"


/**********************************************************************
 * read_line
 * read a line from fd and parse it into a queue of tokens
 **********************************************************************/

queue *read_line(const int fd, queue *q){
  token_t *t;

  if((q=create_queue(q))==NULL){
    syslog(LOG_WARNING, "read_line: create_queue");
    return(NULL);
  }
 
  do{
    if((t=read_token(fd, t))==NULL){
      syslog(LOG_WARNING, "read_line: read_token");
      return(NULL);
    }

    if((q=push(q, (queue_t)t, DESTROY_TOKEN))==NULL){
      syslog(LOG_WARNING, "read_line: push");
      return(NULL);
    }
  }while(!token_is_eol(t));

  return(q);
}


/**********************************************************************
 * queue_to_string
 * convert the contents of a queue of tokens into a string
 * a space ( ) is inserted in the resultant string between each
 * token
 **********************************************************************/

char *queue_to_string(queue *q, char *string){
  token_t *t;
  queue *stack;
  int length;
  char *pos;
  
  if((stack=create_queue(stack))==NULL){
    syslog(LOG_WARNING, "queue_to_string: create_queue");
    return(NULL);
  }

  length=0;
  while(q->first!=NULL){
    if((q=pop(q, (queue_t *)&t))==NULL){
      syslog(LOG_WARNING, "queue_to_string: pop 1");
      return(NULL);
    }

    length+=1+t->n;

    if((stack=push(stack, (queue_t)t, DESTROY_TOKEN))==NULL){
      syslog(LOG_WARNING, "queue_to_string: push");
      return(NULL);
    }
  }

  length--;
  destroy_queue(q, DESTROY_TOKEN);

  if((string=(char*)malloc(sizeof(char)*length))==NULL){
    syslog(LOG_WARNING, "queue_string: malloc");
    destroy_queue(stack, DESTROY_TOKEN);
    return(NULL);
  }

  pos=string;
  while(stack->first!=NULL){
    if((stack=pop(stack, (queue_t *)&t))==NULL){
      syslog(LOG_WARNING, "queue_to_string: pop 2");
      free(string);
      return(NULL);
    }
    
    strncpy(pos, t->buf, t->n);
    pos+=t->n;
    *pos++=' ';
  }
  
  destroy_queue(stack, DESTROY_TOKEN);
  *--pos='\0';

  return(string);
}
